<?php

/** 
 * Protocol abstraction for pyramid-framework. It may look as total bloat, but
 * really it is very useful.
 * 
 * @package CTL
 * @author Emilis Dambauskas (emilis@gildija.lt)
 * @copyright 20022005 Emilis Dambauskas under {@link http://opensource.org/licenses/mit-license.php MIT license}
 * @version $Id: ctlHttpProtocol.class.php,v 1.5 2004/08/19 13:15:40 lunaticlt Exp $
 * @class ctlHttpProtocol 
 */
class ctlHttpProtocol
{
	/**
	 * Link to the {@link $db database}
	 * @attribute private object $db
	 */
	var $db;
	
	/**
	 * Client vars
	 * @attribute private array $client
	 */
	var $client;
	
	/**
	 * Session vars
	 * @attribute private array $session
	 */
	var $session;
	
	/**
	 * Server vars
	 * @attribute private array $server
	 */
	var $server;
	
	/**
	 * Request vars
	 * @attribute private array $request
	 */
	var $request;
	
	/**
	 * Protocol vars
	 * @attribute private array $protocol
	 */
	var $protocol;
	
	/**
	 * Commands (HTTP headers)
	 * @attribute private array $commands
	 */
	var $commands;
	
	/**
	 * Session id
	 * @attribute private string $sid
	 */
	var $sid;
	
	/**
	 * Cookie name
	 * @attribute private string $cookien
	 */
	var $cookien;
	
	/**
	 * Session table name
	 * @attribute private string $stable
	 */
	var $stable;
	
	/**
	 * Session cookie name
	 * @attribute private string $scookie
	 */
	var $scookie;
	
	/**
	 * Session lifetime (defaults to 1200 s = 20 min.)
	 * @attribute private int $slifetime
	 */
	var $slifetime;
	
	
	/**
	 * Constructor. Fills attributes with values either from global {@link $config} or with default values.
	 *
	 * @constructor ctlHttpProtocol   
	 * @use $config  
	 * @use $db 
	 */ 
	function ctlHttpProtocol()
	{
		$this->client = Array();
		$this->server = Array();
		$this->session = Array();
		$this->request = Array();
		$this->protocol = Array();
		$this->commands = Array();
		$this->sid = '';
		
		$this->db = &$GLOBALS['db'];
		$this->db = &loadObject('adodb');
		
		if (isset($GLOBALS['config']['protocol']))
		{
			$cfg = $GLOBALS['config']['protocol'];
			$this->cookien = $cfg['cookie_name'];
			$this->stable = $cfg['session_table'];
			$this->scookie = $cfg['session_cookie'];
			$this->slifetime = $cfg['session_lifetime'];
		}
		else
		{
			$this->cookien = 'ctl_cookie';
			$this->stable = 'ctl_sessions';
			$this->scookie = 'ctl_scookie';
			$this->slifetime = 1200; // 20 min.
		}
	}
	
	/**
	 * Sends an error to the {@link $errors error handler}
	 *
	 * @method private error
	 * @return boolean always FALSE 
	 * @param string $msg Error message  
	 * @use $errors  
	 */ 
	function error($msg)
	{
		$GLOBALS['errors']->add(get_class($this).': '.$msg);
		return FALSE;
	}
	
	/**
	 * Strips slashes recursively
	 *
	 * @method private stripSlashes  
	 * @param ref mixed $var Original variable**  
	 */  
	function stripSlashes(&$var)
	{
	    if (is_array($var)) 
            foreach ($var as $key => $val)
                $this->stripSlashes($var[$key]);

        else if (is_string($var))
            $var = stripslashes($var);
    }
	
	/**
	 * Starts sessions, fills variable arrays
	 *
	 * @method public start
	 */  
	function start()
	{
        // Fix magic quotes.
        if (get_magic_quotes_gpc())
        {
            $keys = array_keys($_GET);
            foreach ($keys as $k) 
        	    $this->stripSlashes($_GET[$k]);
        	
        	$keys = array_keys($_REQUEST);
            foreach ($keys as $k) 
        	    $this->stripSlashes($_REQUEST[$k]);
        	
        	$keys = array_keys($_COOKIE);
            foreach ($keys as $k) 
        	    $this->stripSlashes($_COOKIE[$k]);
        }
        		
		$gvars = explode('?', $_SERVER['REQUEST_URI']);
		$gvars = explode('&', @$gvars[1]);
		foreach ($gvars as $gv)
		{
			$gv = explode('=', $gv);
			if (!isset($_GET[$gv[0]]))
				$_GET[$gv[0]] = urldecode(@$gv[1]);
		}
		
		$this->request = &$_REQUEST; //array_merge($_GET, $_POST);
		
		if (!$this->loadSession())
			$this->startSession();
		
		if (isset($_COOKIE[$this->cookien]))
			$this->client = unserialize(stripslashes($_COOKIE[$this->cookien]));
		
		$this->server = $_SERVER;
	}
	
	/**
	 * Outputs data, saves session, etc.
	 *
	 * @method public output  
	 * @param string $content Data for outputing  
	 */  
	function output($content)
	{
		if (!headers_sent())
		{
			$_COOKIE[$this->cookien] = serialize($this->client);
			@setcookie($this->cookien, serialize($this->client), 86400*30);
		
			foreach ($this->commands as $hdr)
				header($hdr);
		}
		$this->saveSession();

		echo $content;
	}
	
	/**
	 * Outputs local system file with correct headers, saves session, etc.
	 *
	 * @method public outputFile  
	 * @return boolean TRUE on success, FALSE on failure
	 * @param string $fname file name (NOTE: '../' is replaced for security reasons)  
	 * @param string $disp Content-Disposition type (default attachment)
	 * @use ENGINE_DIR  
	 * @use ctlMimeType 
	 */ 
	function outputFile($fname, $disp = 'attachment')
	{
		if (!headers_sent())
		{
			$_COOKIE[$this->cookien] = serialize($this->client);
			setcookie($this->cookien, serialize($this->client), 86400*30);
		
			foreach ($this->commands as $hdr)
				header($hdr);
		}
		
		$this->saveSession();
		
		$fname = str_replace('../', '', $fname);
		if (!file_exists($fname))
			$this->output('ERROR: File not found.');
		else
		{
			$fsize = filesize($fname);
			
			require_once(ENGINE_DIR.'ctlMimeType.class.php');
			$mime = &new ctlMimeType(ENGINE_DIR.'mime.types');
			
			$fmime = $mime->getType($fname);
			
			if (!$fp = fopen($fname, "rb"))
				return $this->error('fopen: '.$fname.' failed in method outputFile.');

			header('Accept-Ranges: bytes');
			header('Content-Length: '.$fsize);
			
			if ($fmime)
				header('Content-Type: '.$fmime);
			$ffname = array_pop(explode('/', $fname));
			header('Content-Disposition: '.$disp.'; filename='.$ffname);
			
			// file last modified info:
			header('Expires: '.gmdate('D, d M Y H:i:s', time() + 86400).' GMT', 'header');
			header('Last-Modified: '.gmdate('D, d M Y H:i:s', filemtime($fname)).' GMT');
			header('Cache-Control: private');
			header('Connection: close');
			
			if (!fpassthru($fp))
			{
				@fclose($fp);	
				return $this->error('fpassthru '.$fname.' failed in method outputFile.');
			}
			@fclose($fp);
			return TRUE;
		}
	}
	
	/**
	 * Use this instead of header() function - adds header to a stack (so it won't 
	 * conflict with cookies)
	 *
	 * @method public sendCommand  
	 * @param string $cmd HTTP header  
	 */  
	function sendCommand($cmd)
	{
		array_push($this->commands,$cmd);
	}

//----------------- CLIENT METHODS --------------------------------------------

	/**
	 * Sets variable on client (cookie)
	 *
	 * @method public setClientVar
	 * @return boolean TRUE 
	 * @param string $name variable name	 
	 * @param optional mixed $value Variable value  
	 */  
	function setClientVar($name, $value = NULL)
	{
		$this->client[$name] = $value;
		return TRUE;
	}
	
	/**
	 * Returns a client var
	 *
	 * @method public getClientVar
	 * @return mixed whatever the variable value is 
	 * @param string $name variable name  
	 */  
	function getClientVar($name)
	{
		return @$this->client[$name];
	}
	
	/**
	 * Removes variable on client
	 *
	 * @method public delClientVar
	 * @return boolean TRUE 
	 * @param string $name variable name  
	 */  
	function delClientVar($name)
	{
		unset($this->client[$name]);
		return TRUE;
	}
	
	/**
	 * Registers an external variable as client variable. Usefull for keeping 
	 * client variable up to date automatically.
	 *
	 * @method public regClientVar
	 * @return boolean TRUE 
	 * @param string $name variable name 
	 * @param ref mixed $var Reference to external variable  
	 */  
	function regClientVar($name, &$var)
	{
		$this->client[$name] = &$var;
		return TRUE;
	}
	
	/**
	 * Removes all client variables
	 *
	 * @method public emptyClientVars   
	 */  
	function emptyClientVars()
	{
		$this->client = Array();
	}

//----------------- SESSION METHODS -------------------------------------------

	/**
	 * Starts a session
	 *
	 * @method public startSession
	 * @return boolean TRUE on success, FALSE on failure  
	 */  
	function startSession()
	{
		mt_srand((double)microtime()*1234567);
		$this->sid = md5(mt_rand());
		setcookie($this->scookie, $this->sid, time()+100000, '/');
		$sql = 'INSERT INTO '.$this->stable.' (id, last, data) VALUES('.$this->db->Quote($this->sid).', '.time().', '.$this->db->Quote(serialize($this->session)).')';
		if ($this->db->query($sql))
			return TRUE;
		else
			return $this->error('Could not insert session into db. SQL: '.$sql);
	}
	
	/**
	 * Saves session status and variables
	 *
	 * @method public saveSession
	 * @return boolean TRUE on success, FALSE on failure  
	 */  
	function saveSession()
	{
		$sql = 'UPDATE '.$this->stable.' SET last='.time().', data='.$this->db->Quote(serialize($this->session)).' WHERE id='.$this->db->Quote($this->sid);
		if (!$this->db->query($sql))
			return $this->error('Failed to save session '.$this->sid.'. SQL: '.$sql);
		else
			return TRUE;
	}
	
	/**
	 * Loads session status and variables
	 *
	 * @method public loadSession
	 * @return boolean TRUE on success, FALSE on failure  
	 */  
	function loadSession()
	{
		$gtime = time() - $this->slifetime;
		$this->db->query('DELETE FROM '.$this->stable.' WHERE last < '.$gtime);
		
		if (!isset($_COOKIE[$this->scookie]))
			return FALSE;
				
		$sql = 'SELECT * FROM '.$this->stable.' WHERE id='.$this->db->Quote($_COOKIE[$this->scookie]);
		if (!$temp = $this->db->getRow($sql))
			return FALSE;
		else
		{
			$this->sid = $_COOKIE[$this->scookie];
			$this->session = unserialize($temp['data']);
			$this->db->query('UPDATE '.$this->stable.' SET last='.time().' WHERE id='.$this->db->Quote($this->sid));
			return TRUE;
		}
	}
	
	/**
	 * Ends session
	 *
	 * @method public endSession
	 * @return boolean TRUE on success, FALSE on failure  
	 */  
	function endSession()
	{
		$this->session = array();
		if (!$this->db->query('DELETE FROM '.$this->stable.' WHERE id='.$this->db->Quote($this->sid)))
				return $this->error('Couldnot end session '.$this->sid.'.');
		else
		{
			$this->sid = '';
			setcookie($this->scookie);
			return TRUE;
		}
	}
	
	/**
	 * Sets session variable
	 *
	 * @method public setSessionVar
	 * @return boolean TRUE 
	 * @param string $name variable name 
	 * @param optional mixed $value variable value  
	 * @param optional int $serialized set to TRUE if the value should be kept in string representation internally
	 */  
	function setSessionVar($name, $value = NULL, $serialized = FALSE)
	{
		if ($serialized)
		    $this->session[$name] = serialize($value);
		else
		    $this->session[$name] = $value;
		return TRUE;
	}
	
	/**
	 * Returns session variable value
	 *
	 * @method public getSessionVar
	 * @return mixed whatever the value is 
	 * @param string $name variable name  
	 * @param optional int $serialized set to TRUE if the value was kept in string representation internally
	 */  
	function getSessionVar($name, $serialized = FALSE)
	{
		if ($serialized)
		    return @unserialize($this->session[$name]);
		else
		    return @$this->session[$name];
	}
	
	/**
	 * Removes variable from session
	 *
	 * @method public delSessionVar
	 * @return boolean TRUE 
	 * @param string $name variable name  
	 */  
	function delSessionVar($name)
	{
		unset($this->session[$name]);
		return TRUE;
	}
	
	/**
	 * Registers an external variable as session variable. Usefull for keeping 
	 * session variable up to date automatically.
	 * 
	 * @method public regSessionVar
	 * @return boolean TRUE 
	 * @param string $name variable name 
	 * @param ref mixed $var variable  
	 */  
	function regSessionVar($name, &$var)
	{
		$this->session[$name] = &$var;
		return TRUE;
	}
	
	/**
	 * Removes all variables from session
	 *
	 * @method public emptySessionVars   
	 */  
	function emptySessionVars()
	{
		$this->session = Array();
	}

//----------------- SERVER METHODS --------------------------------------------

	/**
	 * Returns server variable value
	 *
	 * @method public getServerVar
	 * @return mixed whatever the variable value is 
	 * @param string $name variable name  
	 */  
	function getServerVar($name)
	{
		return @$this->server[$name];
	}

//----------------- REQUEST METHODS -------------------------------------------

	/**
	 * Returns request variable value
	 *
	 * @method public getRequestVar
	 * @return mixed whatever the value is 
	 * @param string $name variable name  
	 */  
	function getRequestVar($name)
	{
		return @$this->request[$name];
	}
	
	/**
	 * Returns all request variables as an associative array
	 *
	 * @method public getAllRequestVars
	 * @return array associative array of variables indexed by their names  
	 */  
	function getAllRequestVars()
	{
		return $this->request;
	}

}
?>