<?php

/** 
 * Class for mapping HTTP requests to objects and methods.
 * 
 * @package CTL
 * @author Emilis Dambauskas (emilis@gildija.lt)
 * @copyright 2005 Emilis Dambauskas under {@link http://opensource.org/licenses/mit-license.php MIT license}
 * @version $Id$
 * @class ctlRequestMapper 
 */
class ctlRequestMapper
{
    var $name;
    var $config;
    
    var $allow_config_parsed;
    var $allow;
    var $deny;
    var $order;
    
    
    /**
     *
     */
    function ctlRequestMapper($obj_fullname, $config)
    {
        $this->name = $obj_fullname;
        $this->config = $config;
        
        $this->allow_config_parsed = FALSE;
        
        // both parameters should be defined in config if you want to override
        // the default object and action.s
        if (!@$config['default_object'] || !@$config['default_action'])
        {
            $this->config['default_object'] = 'Site';
            $this->config['default_action'] = 'showIndex';
        }
        
        // set allow parameters to default empty values:
        $this->allow = array('star' => array(), 'obj' =>  array(), 'action' => array());
        $this->deny = array('star' => array(), 'obj' =>  array(), 'action' => array());
        $this->order = $this->config['allow_deny_order'];
    }
    
    
    /**
     *
     */
    function mapRequest()
    {
        $passed = TRUE;
        
        // --- 1. create $obj_fullname and $action params:
        // 1.1. try object&action
        if (@$_REQUEST['object'] && @$_REQUEST['action'])
        {
            $obj_fullname = $_REQUEST['object'];
            $action = $_REQUEST['action'];
        }
        // 1.2. try script
        else if (@$_REQUEST['script'])
        {
            $obj_fullname = 'Site.Site';
            $action = 'showScript';
        }
        // 1.3. try page
        else if (@$_REQUEST['page'])
        {
            $obj_fullname = 'Site.Site';
            $action = 'showPage';
        }
        // 1.4. use default
        else if (@$this->config)
        {
            $obj_fullname = $this->config['default_object'];
            $action = $this->config['default_action'];
        }
        // 1.5. return error
        else
        {
            $passed = FALSE;
        }
        
        // 1.6. fix obj_fullname:
        if ($passed)
            $obj_fullname = fixObjectFullname($obj_fullname);
        
        // --- 2. check for permissions to call the object from web
        if ($passed && !$this->isCallAllowed($obj_fullname, $action))
            $passed = FALSE;
        
        
        // --- 3. load object and call method on it:
        // 3.1. load object:
        if ($passed)
        {
            $obj = &loadObject($obj_fullname);
        
            if (!is_object($obj))
                $passed = FALSE;
        }
        
        // 3.2. check if object loaded and if it has correct method:
        if ( $passed && !method_exists($obj, $action) )
            $passed = FALSE;
        
        // 3.3. check if object has allowed for web method list:
        if ($passed && method_exists($obj, 'listMethodsAllowedForWeb'))
        {
            $allowed_methods = $obj->listMethodsAllowedForWeb();
            if (!in_array($action, $allowed_methods))
                $passed = FALSE;
        }
        
        // 3.4. call method on object and return result:
        if ($passed)
            return call_user_func(array(&$obj, $action));
        else
        {
            // load site
            $site = &loadObject('Site');
            
            // if site could not be loaded, die with error
            if (!is_object($site))
            {
                die('Failed to report error. Loading Site object failed in '.basename(__FILE__).'.');
                return FALSE;
            }
            // if site is loaded return error page
            else
            {
                return $site->showError(404);
            }
        }
    } // end of method mapRequest()
    
    
    /**
     *
     */
    function redirect($call, $params = NULL)
    {
        $call = explode(':', $call);
        
        $url = 'index.php?object='.$call[0].'&action='.$call[1];
        
        header('Location: '.$url);
        exit();
    }
    
    
    /**
     *
     */
    function isCallAllowed($obj_fullname, $action)
    {
        // parse allow config if not parsed:
        if (!$this->allow_config_parsed)
            $this->parseAllowConfig();
        
        // check according to order:
        if ($this->order == 'deny,allow')
        {
            if ($this->searchForAllow($obj_fullname, $action))
                return TRUE;
        }
        else
        {
            if ($this->searchForDeny($obj_fullname, $action))
                return FALSE;
            if ($this->searchForAllow($obj_fullname, $action))
                return TRUE;
        }
        
        // default is "not allowed":
        return FALSE;
    }
    
    
    /**
     *
     */
    function parseAllowConfig()
    {
        // set variables to empty values:
        $this->allow = array('star' => array(), 'obj' =>  array(), 'action' => array());
        $this->deny = array('star' => array(), 'obj' =>  array(), 'action' => array());
        $this->order = $this->config['allow_deny_order'];
        
        // parse allow string:
        $a = explode(',', $this->config['allow']);
        for ($i=0,$s=sizeof($a); $i < $s; $i++)
        {
            $spos = strpos($a[$i], '*');
            if ($spos !== FALSE)
                $this->allow['star'][] = substr($a[$i], 0, $spos);
            else if (strpos($a[$i], ':'))
                $this->allow['action'][] = $a[$i];
            else
                $this->allow['obj'][] = $a[$i];
        }
        
        // parse deny string:
        $d = explode(',', $this->config['deny']);
        for ($i=0,$s=sizeof($d); $i < $s; $i++)
        {
            $spos = strpos($d[$i], '*');
            if ($spos !== FALSE)
                $this->deny['star'][] = substr($d[$i], 0, $spos);
            else if (strpos($d[$i], ':'))
                $this->deny['action'][] = $d[$i];
            else
                $this->deny['obj'][] = $d[$i];
        }
        
        // remember that the config is parsed:
        $this->allow_config_parsed = TRUE;
    }
    
    
    /**
     *
     */
    function searchForAllow($obj_fullname, $action)
    {
        $call = $obj_fullname.':'.$action;
        
        if (in_array($call, $this->allow['action']))
            return TRUE;
        else if (in_array($obj_fullname, $this->allow['obj']))
            return TRUE;
        else
        {
            foreach ($this->allow['star'] as $star)
            {
                if ($star === substr($call, 0, strlen($star)))
                    return TRUE;
            }
        }
        
        // default not found:
        return FALSE;
    }
    
    
    /**
     *
     */
    function searchForDeny($obj_fullname, $action)
    {
        $call = $obj_fullname.':'.$action;
        
        if (in_array($call, $this->deny['action']))
            return TRUE;
        else if (in_array($obj_fullname, $this->deny['obj']))
            return TRUE;
        else
        {
            foreach ($this->deny['star'] as $star)
            {
                if ($star === substr($call, 0, strlen($star)))
                    return TRUE;
            }
        }
        
        // default not found:
        return FALSE;
    }
}

?>