<?php

/** 
 * Site.Site object class.
 * 
 * @package CTL
 * @author Emilis Dambauskas  (emilis@gildija.lt)
 * @copyright MIT license (opensource).
 * @version $Id$
 * @class Site
 */
class ctlSimpleSite
{
    /**
     * Object full name.
     * @attribute private string $name
     */
    var $name;
    
    /**
     * Configuration array.
     * @attribute private array $conrfig
     */
    var $config;
    
    
    var $page_dir;
    var $script_dir;
    var $tpl_dir;
    
    /**
     *
     */
    function ctlSimpleSite($obj_fullname, $config)
    {
        // general config
        $this->name = $obj_fullname;
        $this->config = $config;
        
        // tpl_dir setting
        if (!@$config['tpl_dir'])
        {
            if (defined('TPL_DIR'))
                $this->tpl_dir = TPL_DIR;
            else
                $this->tpl_dir = SYSTEM_DIR.'/tpl';
        }
        
        // page_dir setting
        if (!@$config['page_dir'])
            $this->page_dir = SYSTEM_DIR.'/pages';
        else
            $this->page_dir = $config['page_dir'];
        
        // script_dir setting
        if (!@$config['script_dir'])
            $this->script_dir = SYSTEM_DIR.'/scripts';
        else
            $this->script_dir = $config['script_dir'];
    }
    
    
    /**
     *
     */
    function showError($msg)
    {
        if ($msg == 404)
            $msg = 'Page not found.';
        
        $tpl = &loadObject('ctl.tpl');
        $tpl->setVar('message', $msg);
        return $tpl->process($this->tpl_dir.'/site-error.tpl');
    }
    
    
    /**
     *
     */
    function showContent($content)
    {
        $tpl = &loadObject('ctl.tpl');
        
        if (is_array($content))
            $tpl->setVars($content);
        else
            $tpl->setVar('html', $content);
        
        return $tpl->process($this->tpl_dir.'/site-main.tpl');
    }
    
    
    /**
     *
     */
    function showIndex()
    {
        return $this->showScript('index');
    }
    
    
    /**
     *
     */
    function showPage($page_name = NULL)
    {
        if ($page_name === NULL)
            $page_name = @$_REQUEST['page'];
        
        $file_name = $this->page_dir.'/'.$page_name.'.html';
        
        if (!file_exists($file_name))
        {
            return $this->showError(404);
        }
        else
        {
            // 1. get contents
            $handle = fopen ($file_name, 'rb'); 
            $contents = fread ($handle, filesize ($file_name)); 
            fclose ($handle);

            // 2. strip everything outside body tags
            if ($body1_start = strpos($contents, '<body'))
            {
                $body1_end = strpos($contens, '>', $body_start);
                $body2_start = strpos('</body>', $contents);
                
                $contents = substr($contents, $body1_end, ($body2_start - $body1_end) );
            }
            else if ($body1_start = strpos($contents, '<BODY'))
            {
                $body1_end = strpos($contens, '>', $body_start);
                $body2_start = strpos('</body>', $contents);
                
                $contents = substr($contents, $body1_end, ($body2_start - $body1_end) );
            }
            else
            {
                // For those who use mixed case we do nothing.
                // Also nothing is done for those that have no body tags.
            }
            
            if (preg_match('/<h1>([^<]+)<\/h1>/im', $contents, $matches))
                $contents = array('title' => $matches[1], 'html' => $contents);
            
            // 3. return contents
            return $this->showContent($contents);
        }
    }
    
    
    /**
     *
     */
    function showScript($script_name = NULL)
    {
        if ($script_name === NULL)
            $script_name = @$_REQUEST['script'];
        
        $file_name = $this->script_dir.'/'.$script_name.'.php';
        
        if (!file_exists($file_name))
        {
            return $this->showError('Script "'.$script_name.'" does not exist.');
        }
        else
        {
            $e = error_reporting(E_ALL ^ E_WARNING ^ E_NOTICE);
            ob_start();
            include($file_name);
            $content = ob_get_contents();
            ob_end_clean();
            error_reporting($e);
            
            return $this->showContent($content);
        }
    }
    
    
    
    
}

?>