<?php

/**
 * Loader function for the ADOdb Database Abstraction Library for PHP
 *
 * @function adodb_loader
 * @return ref object ADOdb connection object. 
 * @param string $obj_fullname Object full name. 
 * @param array $obj_config Object config.  
 * @use $config  
 */ 
function &ADOdb_loader($obj_fullname, $obj_config)
{
    global $config;
    
    // 1. If no config given, take config from $config[db]:
    if (!is_array($obj_config) || !sizeof($obj_config))
        $obj_config = $config['db'];
    
    // 2. Try loading adodb library files:
    if (!loadLibrary('/ADOdb/adodb.inc.php'))
        return FALSE;
    
    // 3. Create connection object for the given database type:
    $conn = &ADONewConnection($obj_config['type']);
    
    // 4. Connect to database:
    if (@$obj_config['persistent'])
        $conn->PConnect($obj_config['host'],$obj_config['user'],$obj_config['password'],$obj_config['database']);
    else
        $conn->Connect($obj_config['host'],$obj_config['user'],$obj_config['password'],$obj_config['database']);
    
    // 5. More configuration:
    $conn->debug = @$obj_config['debug'];
    
    // 6. Return connection object:
    return $conn;
}


/**
 * Unloader function for ADOdb connection object.
 *
 * @function adodb_unloader
 * @return boolean Null. 
 * @param ref object ADOdb connection object.
 */  
function ADOdb_unloader(&$obj)  
{
    // 1. Close database connection and return the result:
    return $obj->Close();
}

?>