<?php

/**
 * Loader function for the CTL module objects.
 *
 * @function ctl_loader
 * @return ref object Loaded object. 
 * @param string $obj_fullname Object full name. 
 * @param array $obj_config Object config.  
 * @use $config  
 */ 
function &ctl_loader($obj_fullname, $obj_config)
{
    global $config;
    
    // 1. Get object name from full name:
    $obj_name = getObjectName($obj_fullname);
    
    // 2. Switch through available objects:
    switch ($obj_name)
    {
        //---> ctl.mapper
        case 'mapper':
            if (!loadLibrary('/ctl/ctlRequestMapper.class.php'))
                return FALSE;
            
            $obj = &new ctlRequestMapper($obj_fullname, $config['Mapper']);
            return $obj;
        break;
        
        //---> ctl.tpl
        case 'tpl':
            if (!loadLibrary('/ctl/ctlTpl.class.php'))
                return FALSE;
            
            $tpl_dir = (@$obj_config['tpl_dir']) ? $obj_config['tpl_dir'] : TPL_DIR;
            $obj = &new ctlTpl($tpl_dir, @$obj_config['check_templates']);
            return $obj;
        break;
        
        //---> ctl.tpl2
        case 'tpl2':
            if (!loadLibrary('/ctl/ctlTpl2.class.php'))
                return FALSE;
            
            $obj = &new ctlTpl2($obj_config['tpl_dir'], $obj_config['check_templates']);
            return $obj;
        break;
        
        //---> default (FALSE)
        default:
            return FALSE;
    }
}


/**
 * Unloader function for CTL objects.
 *
 * @function ctl_unloader
 * @return boolean TRUE on success, FALSE on failure. 
 * @param ref object Object that needs unloading.
 */ 
function ctl_unloader(&$obj)
{
    return TRUE;
}

?>